/*
    JCR - Jabber Component Runtime
    Copyright (C) 2003 Paul Curtis
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
$Id: jcr_mio.c,v 1.5 2004/01/05 14:40:00 pcurtis Exp $
 
*/

#include "jcomp.h"

void jcr_mio_close(mio m) {

   if (m == NULL)
      return;

   if (m->gio == NULL)
      return;

   if (m->cb != NULL)
      (*(mio_raw_cb)m->cb)(m, MIO_CLOSED, m->cb_arg, NULL, 0);

   g_mutex_lock(m->gio_lock);
   if (m->gio) {
     log_debug(JDBG, "mio_close() GIO valid, ref count = %d", m->gio->ref_count);
     g_source_remove(m->watch_source);
     g_io_channel_shutdown(m->gio, TRUE, NULL);
     g_io_channel_unref(m->gio);
   } else {
     log_debug(JDBG, "mio_close() GIO == NULL");
   }
   close(m->fd);
   m->gio = NULL;
   g_mutex_unlock(m->gio_lock);
}

void jcr_mio_read_data(GIOChannel *src, GIOCondition cond, gpointer data) {
   mio m = (mio)data;
   char buf[4096];
   GError *gerr = NULL;
   GIOStatus rc;
   int bytes_read;

   if (cond & G_IO_ERR) {
     log_warn(JDBG, "MIO Channel Error: G_IO_ERR");
     return;
   }

   if (cond & G_IO_HUP) {
     log_warn(JDBG, "MIO Channel Error: G_IO_HUP");
     return;
   }

   if (cond & G_IO_NVAL) {
     log_warn(JDBG, "MIO Channel Error: G_IO_NVAL");
     return;
   }


   if (cond & G_IO_IN) {
      memset(buf, 0, 4096);
      rc = g_io_channel_read_chars(src, buf, 4095, &bytes_read, &gerr);
      if (rc != G_IO_STATUS_NORMAL) {
         if (gerr != NULL) {
            log_warn(JDBG, "MIO Channel Error: rc=%d, %d '%s' '%s'", rc,
                     gerr->code, gerr->domain, gerr->message);
            g_error_free(gerr);
         } else {
            log_warn(JDBG, "MIO Channel Error: rc=%d", rc);
         }
         if (m->cb != NULL)
            (*(mio_raw_cb)m->cb)(m, MIO_ERROR, m->cb_arg, NULL, 0);
         jcr_mio_close(m);
         return;
      }
      log_debug(JDBG, "reading from fd=%d, %d bytes", g_io_channel_unix_get_fd(src), bytes_read);
      if (m->cb != NULL)
         (*(mio_raw_cb)m->cb)(m, MIO_BUFFER, m->cb_arg, buf, bytes_read);
   }
}

xmlnode mio_cleanup(mio m) {
   return NULL;
}

void mio_write(mio m, xmlnode x, char *buffer, int len) {
   GIOStatus rc = G_IO_STATUS_NORMAL;
   unsigned int bytes;
   int left;

   if (len == -1)
      len = strlen(buffer);

   left = len;
   while ((left > 0) && (rc == G_IO_STATUS_NORMAL)) {
      rc = g_io_channel_write_chars(m->gio, (buffer+(len - left)), left, &bytes, NULL);
      log_debug(JDBG, "wrote %d bytes (fd=%d)", bytes, g_io_channel_unix_get_fd(m->gio));
      left = left - bytes;

      if (rc != G_IO_STATUS_NORMAL) {
         log_warn(JDBG, "mio_write failed");
         jcr_mio_close(m);
         return;
      }
      g_io_channel_flush(m->gio, NULL);
   }

}

void mio_close(mio m) {

   if (m == NULL)
      return;

   log_debug(JDBG, "mio_close() called");
   jcr_mio_close(m);
   if (m->p != NULL)
      pool_free(m->p);
   m = NULL;
}

void jcr_mio_internal_connect(void *arg) {
   GIOChannel *gio;
   mio new = (mio)arg;
   struct sockaddr_in sa;
   struct hostent *hp;
   struct timeval t;
   fd_set wfds;
   int flag = 1, flags, flag_len, rc;
   guint watch_src;

   /* create a socket to connect with */
   new->fd = socket(AF_INET, SOCK_STREAM,0);

   /* set socket options */
   if(new->fd < 0 || setsockopt(new->fd, SOL_SOCKET, SO_REUSEADDR, (char*)&flag, sizeof(flag)) < 0) {
      log_warn(JDBG, "errno=%d", errno);
      if(new->cb != NULL)
         (*(mio_std_cb)new->cb)(new, MIO_CLOSED, new->cb_arg);

      if(new->fd > 0)
         close(new->fd);
      pool_free(new->p);
      return;
   }

   /* set the socket to non-blocking */
   flags =  fcntl(new->fd, F_GETFL, 0);
   flags |= O_NONBLOCK;
   fcntl(new->fd, F_SETFL, flags);

   sa.sin_addr.s_addr = inet_addr(new->ip);
   if (sa.sin_addr.s_addr == (u_int) - 1) {
      hp = gethostbyname(new->ip);
      if (!hp) {
         log_warn(JDBG, "errno=%d", errno);
         log_warn(JDBG, "unknown host \"%s\"\n", new->ip);
         if(new->cb != NULL)
            (*(mio_std_cb)new->cb)(new, MIO_CLOSED, new->cb_arg);

         if(new->fd > 0)
            close(new->fd);
         if (new->p != NULL)
            pool_free(new->p);
         return;
      }
      memcpy((caddr_t) &sa.sin_addr, hp->h_addr, hp->h_length);
   }

   sa.sin_family = AF_INET;
   sa.sin_port = htons(new->port);
   FD_ZERO(&wfds);
   FD_SET(new->fd, &wfds);

   t.tv_sec = new->timeout;
   t.tv_usec = 0;

   log_debug(JDBG, "Attempting mio_connect to %s:%d", new->ip, new->port);
   rc = connect(new->fd, (struct sockaddr *) &sa, sizeof(sa));
   if ((rc == -1) && (errno != EINPROGRESS)) {
      log_warn(JDBG, "errno=%d", errno);
      if(new->cb != NULL)
         (*(mio_std_cb)new->cb)(new, MIO_CLOSED, new->cb_arg);

      if(new->fd > 0)
         close(new->fd);
      if (new->p != NULL)
         pool_free(new->p);
      return;
   }

   log_debug(JDBG, "Waiting for connect to %s:%d", new->ip, new->port);
   rc = select((new->fd + 1), NULL, &wfds, NULL, &t);
   if (rc == -1) {
      log_warn(JDBG, "errno=%d", errno);
      if(new->cb != NULL)
         (*(mio_std_cb)new->cb)(new, MIO_CLOSED, new->cb_arg);

      if(new->fd > 0)
         close(new->fd);
      if (new->p != NULL)
         pool_free(new->p);
      return;
   }

   flag_len = sizeof(flag);
   rc = getsockopt(new->fd, SOL_SOCKET, SO_ERROR, (char *)&flag, &flag_len);
   if ((rc == -1) || (flag != 0)) {
      log_warn(JDBG, "errno=%d", errno);
      if(new->cb != NULL)
         (*(mio_std_cb)new->cb)(new, MIO_CLOSED, new->cb_arg);

      if(new->fd > 0)
         close(new->fd);
      if (new->p != NULL)
         pool_free(new->p);
      return;
   }

   log_debug(JDBG, "Connected to %s:%d", new->ip, new->port);
   gio = g_io_channel_unix_new(new->fd);
   new->gio = gio;
   g_io_channel_set_encoding(gio, NULL, NULL);
   g_io_channel_set_buffered(gio, FALSE);

   watch_src = g_io_add_watch(gio, (G_IO_IN | G_IO_ERR | G_IO_HUP | G_IO_NVAL), (void *)jcr_mio_read_data, (void *)new);
   new->watch_source = watch_src;

   /* notify the client that the socket is born */
   if(new->cb != NULL)
      (*(mio_std_cb)new->cb)(new, MIO_NEW, new->cb_arg);

}



void mio_connect(char *host, int port, void *cb, void *cb_arg, int timeout, int type) {
   GThread *cthread;
   mio m;
   pool mp;

   mp = pool_new();
   m = (mio)pmalloco(mp, sizeof(_mio));
   m->p = mp;
   m->cb = cb;
   m->cb_arg = cb_arg;
   m->type = type_NORMAL;
   m->state = state_ACTIVE;
   m->ip = pstrdup(mp, host);
   m->port = port;
   m->timeout = timeout;
   m->gio_lock = g_mutex_new();

   cthread = g_thread_create((void *)jcr_mio_internal_connect, (void *)m, FALSE, NULL);
   return;
}
